library(httr)
library(jsonlite)
library(dplyr)
library(DT)
library(stringr)
library(gitlabr)

set_gitlab_connection(
  gitlab_url = "https://gitlab-forge.din.developpement-durable.gouv.fr" ,
  private_token = Sys.getenv("GITLAB_PAT")
)

response <- data.frame()
# on charge les projets du groupe CSD
buildres <- function(i){
  res_proj100 <- GET(
    "https://gitlab-forge.din.developpement-durable.gouv.fr/api/v4/groups/1013/projects",
    add_headers(`PRIVATE-TOKEN` = Sys.getenv("GITLAB_PAT")),
    query = list(
      include_subgroups = "true",
      per_page = 100,
      page = i
    ),
    timeout(120)  # Timeout augmenté pour les projets
  )
  res <- content(res_proj100, as = "text", encoding = "UTF-8") %>% fromJSON()
  return(res)
}
#on réalise une boucle si plus de 100 projets
x <- 1
repeat {
  res <- buildres(x)
  if (length(res) == 0) {
    break
  }
  response <- bind_rows(response, res)
  x <- x + 1
}
# response <- GET(
#   "https://gitlab-forge.din.developpement-durable.gouv.fr/api/v4/groups/1013/projects",
#   add_headers(`PRIVATE-TOKEN` = Sys.getenv("GITLAB_PAT")),
#   query = list(
#     include_subgroups = "true",
#     per_page = 1000
#   ),
#   timeout(120)  # Timeout augmenté pour les projets
# )

# Parse the JSON content
projects <- response

# Extract the project IDs
project_ids <- projects$id

get_project_name <- function(base_url, project_id, private_token) {
  api_url <- paste0(base_url, "/api/v4/projects/", project_id)
  response <- GET(api_url, add_headers("PRIVATE-TOKEN" = private_token))

  if (status_code(response) == 200) {
    project_info <- content(response, as = "parsed", type = "application/json")
    return(data.frame(id = project_id, name = project_info$name))
  } else {
    warning("Impossible de récupérer le projet ID: ", project_id, " - Code: ", status_code(response))
    return(data.frame(id = project_id, name = NA))
  }
}

# Boucle pour récupérer tous les noms de projets
name_gitlab <- bind_rows(lapply(project_ids, get_project_name,
                                base_url = "https://gitlab-forge.din.developpement-durable.gouv.fr",
                                private_token = Sys.getenv("GITLAB_PAT")))
name_gitlab$id <- as.character(name_gitlab$id)

# Variables pour l'API GitLab
base_url <- "https://gitlab-forge.din.developpement-durable.gouv.fr"
private_token <- Sys.getenv("GITLAB_PAT")  # Récupération du token depuis Renviron


# Fonction pour gérer la pagination de l'API GitLab avec une limite stricte à 160 pages, en vérifiant la présence de pages suivantes
# Fonction pour gérer la pagination de l'API GitLab avec une limite stricte à 160 pages
get_all_pages <- function(url, private_token, max_retries = 3, max_pages = 159) {
  all_data <- list()
  page <- 1

  repeat {
    # Vérification de la condition de fin dans la boucle while
    if (page > max_pages) {
      message("Limite de pages atteinte (", max_pages, "). Arrêt de la récupération.")
      break
    }

    retries <- 0
    success <- FALSE
    message("Traitement de l'URL : ", url, " - Page: ", page)  # Afficher l'URL et la page actuelle

    while (!success && retries < max_retries && page <= max_pages) {
      tryCatch({
        response <- GET(
          url,
          query = list(page = page, per_page = 20),
          add_headers("PRIVATE-TOKEN" = private_token),
          timeout(120)
        )

        # Sortie en cas d'erreur HTTP autre que 200
        if (status_code(response) != 200) {
          stop("La requête API a échoué avec le statut: ", status_code(response))
        }

        # Récupération des données JSON
        page_data <- fromJSON(content(response, "text", encoding = "UTF-8"), flatten = TRUE)

        # Si aucune donnée n'est renvoyée, arrêt de la récupération pour ce projet
        if (length(page_data) == 0) {
          message("Aucune donnée sur la page ", page, ". Fin de la récupération pour ce projet.")
          break
        }

        # Ajouter les données de la page à all_data
        all_data <- append(all_data, list(page_data))
        success <- TRUE  # Succès de la récupération
      }, error = function(e) {
        retries <- retries + 1
        message("Erreur de connexion, tentative: ", retries, ". Détails: ", e$message)
        Sys.sleep(2 * retries)
      })
    }

    # Vérification de la présence d'une page suivante dans l'en-tête "Link"
    link_header <- headers(response)[["link"]]
    if (is.null(link_header) || !grepl("rel=\"next\"", link_header)) {
      message("Pas de page suivante. Fin de la récupération pour ce projet.")
      break
    }

    # Passer à la page suivante
    page <- page + 1
  }

  # Combiner toutes les pages récupérées en un seul dataframe
  return(bind_rows(all_data))
}




# Fonction pour extraire le nom du projet à partir de l'URL
extract_project_name <- function(web_url) {
  str_extract(web_url, "(?<=/)[^/]+(?=/-/)")
}

# Fonction pour récupérer les issues d'un projet GitLab
get_gitlab_issues <- function(base_url, project_id, private_token) {
  api_url <- paste0(base_url, "/api/v4/projects/", project_id, "/issues")
  data <- get_all_pages(api_url, private_token)
  data <- data %>%
    mutate(
      project_name = as.character(project_id),
      type = "issue",
      message = as.character(title),
      author = paste0(author.username,"@developpement-durable.gouv.fr")
    )
  return(data %>% select(project_name, type, message, updated_at, author))
}

# Fonction pour récupérer et filtrer les events d'un projet GitLab
get_gitlab_events <- function(base_url, project_id, private_token) {
  api_url <- paste0(base_url, "/api/v4/projects/", project_id, "/events")
  data <- get_all_pages(api_url, private_token)
  data <- data %>%
    mutate(
      project_name = as.character(project_id),

      type = case_when(
        is.null(target_type) | target_type == "" ~ as.character(action_name),
        as.character(action_name) == "joined" ~ as.character(action_name),
        TRUE ~ as.character(target_type)
      ),

      message = case_when(
        # Cas spécifique : MergeRequest avec action_name == "accepted"
        as.character(target_type) == "MergeRequest" & as.character(action_name) == "accepted" ~ as.character(target_title),

        # Cas général : autres MergeRequest ou WikiPage::Meta
        as.character(target_type) %in% c("WikiPage::Meta") ~ as.character(target_title),

        # Si ce n'est pas un cas spécial, garder note.body
        TRUE ~ as.character(note.body)
      ),

      updated_at = created_at,
      author = paste0(author.username,"@developpement-durable.gouv.fr")
    )%>%
    # left_join(name_gitlab, by = c("project_name" = "id")) %>%
    mutate(project_name = name) %>%
    select(-name)
  return(data %>% select(project_name, type, message, updated_at, author))
}

# Fonction pour récupérer les données d'un projet
get_data_for_project <- function(base_url, project_id, private_token) {
  issues_data <- tryCatch(get_gitlab_issues(base_url, project_id, private_token), error = function(e) NULL)
  events_data <- tryCatch(get_gitlab_events(base_url, project_id, private_token), error = function(e) NULL)
  return(bind_rows(issues_data, events_data))
}

# Fonction pour traiter plusieurs projets avec lapply
get_data_from_multiple_projects <- function(base_url, project_ids, private_token) {
  all_data <- lapply(project_ids, function(project_id) {
    get_data_for_project(base_url, project_id, private_token)
  })
  return(bind_rows(all_data))
}

###########################################################
process_projects <- function(project_ids) {
  result_list <- lapply(project_ids, function(project_id) {
    # Récupérer les branches du projet
    branch <- gl_list_branches(project_id)$name

    # Récupérer les commits pour chaque branche
    branch_results <- lapply(branch, function(b) {
      data <- gl_get_commits(project_id, ref_name = b, max_page = NA)
      Sys.sleep(1)
      data <- data %>% mutate(branche = b, project_id = project_id)
      return(data)
    })

    # Combiner les résultats pour un seul projet
    bind_rows(branch_results)
  })

  # Combiner tous les projets en un seul dataframe
  final_result <- bind_rows(result_list)

  # Ajouter une colonne pour indiquer les doublons (en excluant "branche" et "project_id")
  final_result <- final_result %>%
    mutate(is_duplicate = duplicated(select(., -c(branche, project_id))))

  return(final_result)
}

final_result <- process_projects(project_ids)
final_result$project_id <- as.character(final_result$project_id)

final_result <- final_result %>% mutate(author = committer_email,type = "commit")
final_result <- final_result %>% rename( "updated_at" = 'committed_date', "project_name" = 'project_id'  )
ff <- final_result %>% filter(is_duplicate == FALSE) %>% select(project_name,type , message ,updated_at,author)
###########################################################

# Récupérer les données pour tous les projets
all_data_forge <- get_data_from_multiple_projects(base_url, project_ids, private_token)

all_data_forge <- all_data_forge %>%
  filter(!(is.na(all_data_forge$type) | is.na(all_data_forge$message)) | all_data_forge$type == "joined"|all_data_forge$type == "WikiPage::Meta")
all_data_forge <- bind_rows(ff,all_data_forge)
all_data_forge <- all_data_forge %>%
  mutate(origine ="Gitlab_Forge")

#data.frame avec les groupes et le nom pour left join-----
response_df <- data.frame(
  id = as.character(response$id),
  name = response$name,
  groupe = response$namespace$name,
  topics = sapply(response$topics, function(x) paste(x, collapse = ", ")) # on degroupe et regroupe ?? mais ca fonctionne
)
# Joindre les deux jeux de données
all_data_forge <- all_data_forge %>%
  left_join(response_df, by = c("project_name" = "id")) # Faire un premier left_join() sur la colonne name
all_data_forge$author <-  tolower(all_data_forge$author)

# sauvegarde du jeu de donnée-----
all_data_forge <- all_data_forge %>% select(name , type, message , updated_at , author , topics, origine,groupe)
save(all_data_forge, file = "gitlab_forge.RData")
