server <- function(input, output, session) {
  # Stocker les valeurs réactives
  r <- shiny::reactiveValues(
    filteredByDate = NULL,  # Filtrage uniquement par date
    filteredData = NULL     # Filtrage final après application de tous les filtres

  )

  # 1️⃣ Filtrage prioritaire par date ----
  shiny::observeEvent(input$daterange, {
    shiny::req(input$daterange)  # Vérifier que la sélection est valide

    # Filtrage initial sur la plage de dates
    r$filteredByDate <- all_data %>%
      dplyr::filter(
        updated_at >= as.Date(input$daterange[[1]]) &
          updated_at <= as.Date(input$daterange[[2]])
      )
    # Mise à jour dynamique des autres filtres en fonction des données filtrées par date
    updateFilters()
  })

  # 2️⃣ Mise à jour dynamique des filtres ----
  updateFilters <- shiny::reactive({
    shiny::req(r$filteredByDate)  # Vérifier que les données sont disponibles
    shiny::isolate({
      shiny::updateSelectizeInput(session, "project_name",
                           choices = unique(r$filteredByDate$project_name),
                           selected = input$project_name)

      shiny::updateSelectizeInput(session, "etiquette",
                           choices = r$filteredByDate$etiquette %>%
                             strsplit(split = ",") %>%
                             unlist() %>%
                             stats::na.omit() %>%
                             .[. != ""] %>%
                             trimws() %>%
                             unique(),
                           selected = input$etiquette)

      shiny::updateSelectizeInput(session, "auteur",
                           choices = unique(r$filteredByDate$auteur),
                           selected = input$auteur)

      shiny::updateSelectizeInput(session, "categorie",
                           choices = unique(r$filteredByDate$type),
                           selected = input$categorie)

      shinygouv::updateCheckboxGroupInput_dsfr( inputId = "re_code",
                                     label = "Type d'évènement :",
                                     choices = unique(r$filteredByDate$re_code),
                                     selected = input$re_code,
                                     inline = TRUE)
    })
  })
  # 3️⃣ Filtrage final en fonction des autres filtres ----
  shiny::observe({
    shiny::req(r$filteredByDate)  # Vérifier que les données de base sont filtrées par date

    r$filteredData <- r$filteredByDate %>%
      dplyr::filter(
        (if (length(input$project_name)) project_name %in% input$project_name else TRUE) &
          (if (length(input$etiquette)) sapply(input$etiquette, function(t) grepl(t, etiquette)) %>% rowSums() > 0 else TRUE) &
          (if (length(input$auteur)) auteur %in% input$auteur else TRUE) &
          (if (length(input$re_code)) re_code %in% input$re_code else TRUE) &
          (if (length(input$categorie)) type %in% input$categorie else TRUE)
      )

    # Mettre à jour les filtres restants
    updateFilters()
  })

  # 4️⃣ Réinitialisation des filtres ----
  shiny::observeEvent(input$reset, {
    r$filteredByDate <- all_data %>%
      dplyr::filter(
        updated_at >= as.Date(input$daterange[[1]]) &
          updated_at <= as.Date(input$daterange[[2]])
      )
    r$filteredData <- r$filteredByDate  # Reset total
    shiny::updateSelectizeInput(session, "project_name", selected = NULL)
    shiny::updateSelectizeInput(session, "etiquette", selected = NULL)
    shiny::updateSelectizeInput(session, "auteur", selected = NULL)
    shiny::updateSelectizeInput(session, "categorie", selected = NULL)
    shinygouv::updateCheckboxGroupInput_dsfr( "re_code", selected = NULL,inline = TRUE)
    DT::updateFilters()
  })

  # 5️⃣ Graphique interactif ----
  output$filteredPlot2 <- plotly::renderPlotly({
    shiny::req(r$filteredData)  # Vérifier qu'on a des données filtrées
    p <- ggplot2::ggplot(r$filteredData, ggplot2::aes(x = updated_at, y = as.factor(project_name))) +
      ggplot2::geom_point(ggplot2::aes(color = re_code, text = paste(updated_at, message, sep = "\n")),
                          size = 3, alpha = 0.5) +
      ggplot2::labs(shape = "Événement", color = "Événement", x = "") +
      ggplot2::scale_x_datetime(timezone = "Europe/Paris") +
      gouvdown::theme_gouv() +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 0, hjust = 1),
        axis.title.y = ggplot2::element_blank()
      )

    plotly::ggplotly(p, tooltip = "text", dynamicTicks = TRUE)    %>%
      plotly::config(locale = "fr", displaylogo = FALSE)
  })

  # 6️⃣ Graphique des indicateurs ----
  output$bar_chart <- plotly::renderPlotly({
    shiny::req(r$filteredData)

    percent_time <- as.numeric(max(r$filteredData$updated_at) - min(r$filteredData$updated_at)) /
      as.numeric(max(all_data$updated_at) - min(all_data$updated_at)) * 100

    # Création du dataset
    data <- data.frame(
      dataset = c("Commit", "Projet", "Heure"),
      total = c(sum(r$filteredData$type == "commit"),
                length(unique(r$filteredData$project_name)),
                percent_time)
    )
    plotly::ggplotly(
      ggplot2::ggplot(data, ggplot2::aes(x = dataset, y = total, fill = dataset)) +
        ggplot2::geom_bar(stat = "identity") +
        ggplot2::coord_flip() +
        ggplot2::scale_fill_manual(values = c("#3498db", "#e74c3c", "#2ecc71")) +
        ggplot2::labs(
          x = "Comparaison",
          y = "Pourcentage / Nombre",
          title = "Comparaison des Commits, Projets et Temps"
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(legend.position = "none") +
        ggplot2::geom_text(
          ggplot2::aes(label = paste0(round(total, 1),"%")),
          hjust = -0.2
        )
    ) %>%
      plotly::config(locale = "fr",
                     displaylogo = FALSE)
  })

  output$nb_projet <- shiny::renderText({
    shiny::req(r$filteredData)
    paste("Nombre de projets visualisés",length(unique(r$filteredData$project_name)),sep = " : ")
  })
  output$nb_commit <- shiny::renderText({
    shiny::req(r$filteredData)
    paste("Nombre de commits",sum(r$filteredData$type == "commit"),sep = " : ")
  })
  output$nb_temps <- shiny::renderText({
    shiny::req(r$filteredData)
    paste("Estimation du temps passé sur la période :",sep = " ",round(difftime(
      max(r$filteredData$updated_at) , min(r$filteredData$updated_at),units = "hours")), " heures")
  })


  # 7️⃣ Table des résultats ----
  output$table <- DT::renderDT({
    shiny::req(r$filteredData)
    DT::datatable(r$filteredData, options = list(
      pageLength = 10,
      lengthMenu = c(5, 10, 25, 50),
      autoWidth = TRUE,
      scrollX = TRUE,
      class = "display",
      rownames = FALSE,
      searchHighlight = TRUE
    ))
  })

  # 8️⃣ Graphique des indicateurs -----

  output$min_value <- shiny::renderText({
    shiny::req(input$project_name)  # Vérifie que l'input existe

    # Nombre de projets sélectionnés
    selected_projects <- length(unique(input$project_name))

    # Condition : Si 1 seul projet est sélectionné
    if (selected_projects == 1) {
      # Filtrer les données pour ce projet spécifique
      projet_data <- all_data %>%
        dplyr::filter(project_name == input$project_name)

      min_value <- min(projet_data$updated_at, na.rm = TRUE)

      paste("Date de création du projet : ", format(min_value, "%d %B %Y, %H:%M"))
    } else {
      "Sélectionnez un seul projet pour voir la date de création."
    }
  })


  output$temps <- shiny::renderText({
    shiny::req(input$project_name)  # Vérifie que l'input existe

    # Nombre de projets sélectionnés
    selected_projects <- length(unique(input$project_name))
    # Condition : Si 1 seul projet est sélectionné
    if (selected_projects == 1) {
      # Filtrer les données pour ce projet spécifique
      projet_data <- all_data %>%
        dplyr::filter(project_name == input$project_name)

      temps <- difftime(max(projet_data$updated_at), min(projet_data$updated_at), units = "secs")

      jours <- as.numeric(temps, units = "days") %/% 1
      heures <- (as.numeric(temps, units = "hours") %% 24) %/% 1
      minutes <- (as.numeric(temps, units = "mins") %% 60) %/% 1
      print(temps)
      paste("Estimation du temps passé : ", jours, "jours,", heures, "heures,", minutes, "minutes")
    } else {
      "Sélectionnez un seul projet pour voir l'estimation."
    }
  })

  output$bar <- shiny::renderPlot({
    shiny::req(input$project_name)  # Vérifie que l'input existe

    selected_projects <- length(unique(input$project_name))

    if (selected_projects == 1) {
      # Filtrer les données pour le project_name sélectionné
      filtered_data <- all_data %>%
        dplyr::filter(project_name == input$project_name) %>%
        dplyr::mutate(month = format(as.Date(updated_at), "%Y-%m"))

      # Définir la plage de mois spécifique au projet sélectionné
      first_month <- lubridate::floor_date(min(as.Date(filtered_data$updated_at)), "month")
      last_month <- lubridate::ceiling_date(max(as.Date(filtered_data$updated_at)), "month")

      # Générer tous les mois entre le premier et le dernier
      full_months <- seq(first_month, last_month, by = "month") %>%
        dplyr::tibble(month = format(., "%Y-%m"))

      # Compter les entrées par mois et compléter les mois manquants
      complete_data <- full_months %>%
        dplyr::left_join(filtered_data %>% dplyr::count(month), by = "month") %>%
        tidyr::replace_na(list(n = 0))

      # Créer le bar plot
      ggplot2::ggplot(complete_data, ggplot2::aes(x = month, y = n)) +
        ggplot2::geom_bar(stat = "identity", fill = "#ff7f27", width = 0.7) +
        ggplot2::theme_minimal() +
        ggplot2::scale_x_discrete(
          name = "Mois",
          labels = function(x) format(as.Date(paste0(x, "-01")), "%B %Y")
        ) +
        ggplot2::labs(y = "Nombre d'entrées") +
        gouvdown::theme_gouv() +
        ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 45, hjust = 1))
    }else{
      ggplot2::ggplot() +
        ggplot2::annotate("text", x = 1, y = 1, label = "Sélectionnez un seul projet pour voir la timeline", size = 5) +
        ggplot2::theme_void()
    }
  })
}
